@@ -36,7 +36,8 @@ class nanoCapture(Thread):
     def __init__(self, configs, 
         camera_num: int = 0, 
         res: tuple(int, int) = None, 
-        exposure: float = None):
+        exposure: float = None
+        queue_size: int = 32):
 
         # populate desired settings from configuration file or function call
         ####################################################################
@@ -58,7 +59,7 @@ class nanoCapture(Thread):
         self._flip_method    = configs['flip']
 
         # Threading Queue
-        self.capture         = Queue(maxsize=32)
+        self.capture         = Queue(maxsize=queue_size)
         self.log             = Queue(maxsize=32)
         self.stopped         = True
         self.cam_lock        = Lock()
@@ -129,12 +130,12 @@ class nanoCapture(Thread):
 
                     self.capture.put_nowait((current_time*1000., img_proc))
                 else:
-                    self.log.put_nowait((logging.WARNING, "NanoCap:Capture Queue is full!"))
+                    if not self.log.full(): self.log.put_nowait((logging.WARNING, "NanoCap:Capture Queue is full!"))
 
             # FPS calculation
             if (current_time - last_time) >= 5.0: # update frame rate every 5 secs
                 self.measured_fps = num_frames/5.0
-                self.log.put_nowait((logging.INFO, "NANOCAM:FPS:{}".format(self.measured_fps)))
+                if not self.log.full(): self.log.put_nowait((logging.INFO, "NANOCAM:FPS:{}".format(self.measured_fps)))
                 last_time = current_time
                 num_frames = 0
 
@@ -275,7 +276,7 @@ class nanoCapture(Thread):
         self.cam_open = self.cam.isOpened()
 
         if not self.cam_open:
-            self.log.put_nowait((logging.CRITICAL, "NanoCap:Failed to open camera!"))
+            if not self.log.full(): self.log.put_nowait((logging.CRITICAL, "NanoCap:Failed to open camera!"))
 
     # Camera Routines
     ##################################################################
@@ -315,9 +316,9 @@ class nanoCapture(Thread):
         if self.cam_open:
             with self.cam_lock:
                 os.system("v4l2-ctl -c exposure_absolute={} -d {}".format(val, self.camera_num))
-            self.log.put_nowait((logging.INFO, "NanoCap:Exposure:{}".format(self._exposure)))
+            if not self.log.full(): self.log.put_nowait((logging.INFO, "NanoCap:Exposure:{}".format(self._exposure)))
         else:
-            self.log.put_nowait((logging.ERROR, "NanoCap:Failed to set exposure to{}!".format(val)))
+            if not self.log.full(): self.log.put_nowait((logging.ERROR, "NanoCap:Failed to set exposure to{}!".format(val)))
 
 ###############################################################################
 # Testing
